from django.conf.urls.defaults import patterns, url


urlpatterns = patterns(
    '',
    url(r'^feed/(?P<slug>[\w-]+)/$', 'jmbo.views.object_detail', name='feed_object_detail'),
    url(r'^search/(?P<slug>[\w-]+)/$', 'jmbo.views.object_detail', name='search_object_detail'),
)



import re

from django import template

from django.utils.html import urlize
register = template.Library()


@register.filter(name='tweetify')
def tweetify(value):
    s = urlize(value)
    s = re.sub(r'(@[\w]+)', '<a href="http://twitter.com/\g<1>">\g<1></a>', s)
    return s



from django.conf.urls.defaults import patterns, url, include


urlpatterns = patterns(
    '',

    (
        r'jmbo_twitter/feed/(?P<id>\d+)/fetch-force/$',
        'jmbo_twitter.admin_views.feed_fetch_force',
        {'redirect_to' : '/admin/jmbo_twitter/feed'},
        'feed-fetch-force',
    ),

    (
        r'^jmbo_twitter/feed/(?P<id>\d+)/tweets/$',
        'jmbo_twitter.admin_views.feed_tweets',
        {},
        'feed-tweets',
    ),

    (
        r'jmbo_twitter/search/(?P<id>\d+)/fetch-force/$',
        'jmbo_twitter.admin_views.search_fetch_force',
        {'redirect_to' : '/admin/jmbo_twitter/search'},
        'search-fetch-force',
    ),

    (
        r'^jmbo_twitter/search/(?P<id>\d+)/tweets/$',
        'jmbo_twitter.admin_views.search_tweets',
        {},
        'search-tweets',
    ),

)



from django.contrib import admin
from django.core.urlresolvers import reverse, NoReverseMatch

from jmbo.admin import ModelBaseAdmin

from jmbo_twitter import models


class FeedAdmin(ModelBaseAdmin):
    inlines = []
    list_display = ('title', '_image', 'subtitle', 'publish_on', 'retract_on', \
        '_get_absolute_url', 'owner', 'created', '_actions'
    )

    def _image(self, obj):
        if obj.profile_image_url:
            return '<img src="%s" />' % obj.profile_image_url
        else:
            return ''
    _image.short_description = 'Image'
    _image.allow_tags = True

    def _actions(self, obj):
        # Once a newer version of jmbo is out the try-except can be removed
        try:
            parent = super(FeedAdmin, self)._actions(obj)
        except AttributeError:
            parent = ''

        try:
            url = reverse('feed-fetch-force', args=[obj.id]),
            one = '<a href="%s">Fetch tweets</a>' % url
        except NoReverseMatch:
            one = "Fetch tweets - add jmbo_twitter admin_urls to settings, eg. <code>(r'^admin/', include('jmbo_twitter.admin_urls'))</code>"

        try:
            url = reverse('feed-tweets', args=[obj.id]),
            two = '<a href="%s" target="_blank">View tweets</a>' % url
        except NoReverseMatch:
            two = "View tweets - add jmbo_twitter admin_urls to settings, eg. <code>(r'^admin/', include('jmbo_twitter.admin_urls'))</code>"

        return parent + '<ul><li>' + one + '</li><li>' + two + '</li></ul>'
    _actions.short_description = 'Actions'
    _actions.allow_tags = True


class SearchAdmin(ModelBaseAdmin):
    inlines = []
    list_display = ('title', 'subtitle', 'criteria', 'publish_on', 'retract_on', \
        '_get_absolute_url', 'owner', 'created', '_actions'
    )

    def _actions(self, obj):
        # Once a newer version of jmbo is out the try-except can be removed
        try:
            parent = super(SearchAdmin, self)._actions(obj)
        except AttributeError:
            parent = ''

        try:
            url = reverse('search-fetch-force', args=[obj.id]),
            one = '<a href="%s">Fetch tweets</a>' % url
        except NoReverseMatch:
            one = "Fetch tweets - add jmbo_twitter admin_urls to settings, eg. <code>(r'^admin/', include('jmbo_twitter.admin_urls'))</code>"

        try:
            url = reverse('search-tweets', args=[obj.id]),
            two = '<a href="%s" target="_blank">View tweets</a>' % url
        except NoReverseMatch:
            two = "View tweets - add jmbo_twitter admin_urls to settings, eg. <code>(r'^admin/', include('jmbo_twitter.admin_urls'))</code>"

        return parent + '<ul><li>' + one + '</li><li>' + two + '</li></ul>'
    _actions.short_description = 'Actions'
    _actions.allow_tags = True


admin.site.register(models.Feed, FeedAdmin)
admin.site.register(models.Search, SearchAdmin)



from celery.decorators import periodic_task
from celery.task.schedules import crontab

from jmbo_twitter.models import Feed, Search


@periodic_task(run_every=crontab(hour='*', minute='*/15', day_of_week='*'))
def fetch_feeds():
    for feed in Feed.objects.all():
        feed.fetch(force=True)


@periodic_task(run_every=crontab(hour='*', minute='*/15', day_of_week='*'))
def fetch_searchs():
    for search in Search.objects.all():
        search.fetch(force=True)



from django.conf.urls.defaults import patterns, include


urlpatterns = patterns(
    (r'^jmbo/', include('jmbo.urls')),
    (r'^comments/', include('django.contrib.comments.urls')),
    (r'^jmbo_twitter/', include('jmbo_twitter.urls')),
)



from django.core import management
from django.test import TestCase as BaseTestCase
from django.test.client import Client as BaseClient, RequestFactory
from django.contrib.auth.models import User

from twitter import Status

from jmbo_twitter.models import Feed, Search


class FakeAPI(object):

    def GetUserTimeline(self, screen_name=None, include_rts=None):
        return [Status(text='status')]

    def GetSearch(self, term=None):
        return [Status(text='search')]


class TestCase(BaseTestCase):

    @classmethod
    def setUpClass(cls):
        cls.request = RequestFactory()
        cls.client = BaseClient()

        # Editor
        cls.editor, dc = User.objects.get_or_create(
            username='editor',
            email='editor@test.com'
        )
        cls.editor.set_password("password")
        cls.editor.save()

        # Feed
        obj, dc = Feed.objects.get_or_create(
            title='Twitter', name='twitter',
            owner=cls.editor, state='published',
        )
        obj.sites = [1]
        obj.save()
        cls.feed = obj

        # Search
        obj, dc = Search.objects.get_or_create(
            title='Wtf', criteria='#wtf',
            owner=cls.editor, state='published',
        )
        obj.sites = [1]
        obj.save()
        cls.search = obj

        cls.fakeapi = FakeAPI()

    def test_feed_get_statuses(self):
        statuses = self.feed.get_statuses(self.fakeapi)
        self.assertEqual(len(statuses), 1)

    def test_search_get_statuses(self):
        statuses = self.search.get_statuses(self.fakeapi)
        self.assertEqual(len(statuses), 1)

    def test_feed_detail_view(self):
        response = self.client.get(self.feed.get_absolute_url())
        self.assertEqual(response.status_code, 200)

    def test_search_detail_view(self):
        response = self.client.get(self.search.get_absolute_url())
        self.assertEqual(response.status_code, 200)



import multiprocessing
from setuptools import setup, find_packages

setup(
    name='jmbo-twitter',
    version='0.4.2',
    description='Pull, cache, display tweets.',
    long_description=(open('README.rst', 'r').read() +
                        open('AUTHORS.rst', 'r').read() +
                        open('CHANGELOG.rst', 'r').read()),
    author='Praekelt Foundation',
    author_email='dev@praekelt.com',
    license='BSD',
    url='http://github.com/praekelt/jmbo-twitter',
    packages = find_packages(),
    install_requires = [
        'python-twitter>=1.1',
        'jmbo>=1.1.1',
        'jmbo-foundry>=1.1.20',
        'django-celery',
        'requests',
    ],
    include_package_data=True,
    tests_require=[
        'django-setuptest>=0.1.4',
        'coverage',
    ],
    test_suite="setuptest.setuptest.SetupTestSuite",
    classifiers=[
        "Programming Language :: Python",
        "License :: OSI Approved :: BSD License",
        "Development Status :: 4 - Beta",
        "Operating System :: OS Independent",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
    zip_safe=False,
)



import datetime, twitter
from urllib2 import URLError
import logging

from django.db import models
from django.core.cache import cache
from django.utils.translation import ugettext_lazy as _
from django.conf import settings

from jmbo.models import ModelBase


logger = logging.getLogger('django')


class Status(ModelBase):
    """Purely a wrapper that allows us to use jmbo-foundry's listings for
    tweets."""
    def __init__(self, status):
        # Copy attributes over
        attrs = ('contributors', 'coordinates', 'created_at', \
            'created_at_in_seconds', 'favorited', 'geo', 'hashtags', 'id', \
            'in_reply_to_screen_name', 'in_reply_to_status_id', \
            'in_reply_to_user_id', 'location', 'now', 'place', \
            'relative_created_at', 'retweet_count', 'retweeted', \
            'retweeted_status', 'source', 'text', 'truncated', 'urls', 'user', \
            'user_mentions', 'created_at_datetime')
        for attr in attrs:
            setattr(self, attr, getattr(status, attr))

    @property
    def as_leaf_class(self):
        return self

    def save(self):
        raise NotImplemented


class StatusMixin(object):

    def get_statuses(self, api):
        raise NotImplemented

    def fetch(self, force=False):
        klass_name = self.__class__.__name__
        cache_key = 'jmbo_twitter_%s_%s' % (klass_name, self.id)
        cached = cache.get(cache_key, None)
        if (cached is not None) and not force:
            return cached

        # Get and check settings
        di = getattr(settings, 'JMBO_TWITTER', {})
        ck = di.get('consumer_key')
        cs = di.get('consumer_secret')
        atk = di.get('access_token_key')
        ats = di.get('access_token_secret')
        if not all([ck, cs, atk, ats]):
            logger.error(
                'jmbo_twitter.models.%s.fetch - incomplete settings' \
                    % klass_name
            )
            return []

        # Query twitter taking care to handle network errors
        api = twitter.Api(
            consumer_key=ck, consumer_secret=cs, access_token_key=atk,
            access_token_secret=ats, requests_timeout=10
        )
        try:
            statuses = self.get_statuses(api)
        except (URLError, ValueError, twitter.TwitterError):
            statuses = []
        except Exception, e:
            # All manner of things can go wrong with integration
            logger.error(
                'jmbo_twitter.models.%s.fetch - %s' % (klass_name, e.message)
            )
            statuses = []

        for status in statuses:
            status.created_at_datetime = datetime.datetime.fromtimestamp(
                status.created_at_in_seconds
            )

        if statuses:
            # Only set if there are statuses. Twitter may randomly throttle us
            # and destroy our cache without this check. Cache for a long time
            # incase Twitter goes down.
            cache.set(cache_key, statuses, 86400)

        # Legacy return
        return statuses

    @property
    def fetched(self):
        klass_name = self.__class__.__name__
        cache_key = 'jmbo_twitter_%s_%s' % (klass_name, self.id)
        return cache.get(cache_key, [])

    @property
    def tweets(self):
        class MyList(list):
            """Slightly emulate QuerySet API so jmbo-foundry listings work"""

            @property
            def exists(self):
                return len(self) > 0

        result = []
        for status in self.fetched:
            result.append(Status(status))

        return MyList(result)


class Feed(ModelBase, StatusMixin):
    """A feed represents a twitter user account"""
    name = models.CharField(
        max_length=255,
        unique=True,
        help_text="A twitter account name, eg. johnsmith"
    )
    profile_image_url = models.CharField(
        null=True, editable=False, max_length=255
    )
    twitter_id = models.CharField(max_length=255, default='', editable=False)

    def get_statuses(self, api):
        # Fall back to slug for historical reasons
        statuses = api.GetUserTimeline(
            screen_name=self.name or self.slug, include_rts=True
        )
        return statuses

    def fetch(self, force=False):
        statuses = super(Feed, self).fetch(force=force)

        if statuses:
            # This is a convenient place to set the feed image url
            status = statuses[0]
            changed = False
            if status.user.profile_image_url != self.profile_image_url:
                self.profile_image_url = status.user.profile_image_url
                changed = True
            if status.user.name != self.title:
                self.title = status.user.name
                changed = True
            if changed:
                self.save()

        return statuses


class Search(ModelBase, StatusMixin):
    """A search represents a twitter keyword search"""
    criteria = models.CharField(
        max_length=255,
        unique=True,
        help_text="Search string or a hashtag"
    )

    class Meta:
        verbose_name_plural = _("Searches")

    def get_statuses(self, api):
        return api.GetSearch(self.criteria)



from django.contrib.admin.views.decorators import staff_member_required
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404, render_to_response
from django.template import RequestContext
from django.contrib import messages
from django.utils.translation import ugettext as _

from jmbo_twitter.models import Feed, Search


@staff_member_required
def feed_fetch_force(request, id, redirect_to):
    """Forcibly fetch tweets for the feed"""
    feed = Feed.objects.get(id=id)
    feed.fetch(force=True)
    msg = _("Fetched tweets for %s" % feed.name)
    messages.success(request, msg, fail_silently=True)
    return HttpResponseRedirect(redirect_to)


@staff_member_required
def feed_tweets(request, id):
    extra = dict(object=Feed.objects.get(id=id))
    return render_to_response(
        'admin/jmbo_twitter/feed_tweets.html',
        extra,
        context_instance=RequestContext(request)
    )


@staff_member_required
def search_fetch_force(request, id, redirect_to):
    """Forcibly fetch tweets for the search"""
    search = Search.objects.get(id=id)
    search.fetch(force=True)
    msg = _("Fetched tweets for %s" % search.criteria)
    messages.success(request, msg, fail_silently=True)
    return HttpResponseRedirect(redirect_to)


@staff_member_required
def search_tweets(request, id):
    extra = dict(object=Search.objects.get(id=id))
    return render_to_response(
        'admin/jmbo_twitter/feed_tweets.html',
        extra,
        context_instance=RequestContext(request)
    )
